<?php

namespace Litecms\Movie\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Litecms\Movie\Forms\Movie as MovieForm;
use Litecms\Movie\Http\Requests\MovieRequest;
use Litecms\Movie\Interfaces\MovieRepositoryInterface;
use Litecms\Movie\Repositories\Eloquent\Filters\MovieResourceFilter;
use Litecms\Movie\Repositories\Eloquent\Presenters\MovieListPresenter;

/**
 * Resource controller class for movie.
 */
class MovieResourceController extends BaseController
{

    /**
     * Initialize movie resource controller.
     *
     *
     * @return null
     */
    public function __construct(MovieRepositoryInterface $movie)
    {
        parent::__construct();
        $this->form = MovieForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('litecms.movie.modules'), 'movie', guard_url('movie'));
        $this->repository = $movie;
    }

    /**
     * Display a list of movie.
     *
     * @return Response
     */
    public function index(MovieRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(MovieResourceFilter::class)
            ->setPresenter(MovieListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('movie::movie.names'))
            ->view('movie::movie.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display movie.
     *
     * @param Request $request
     * @param Model   $movie
     *
     * @return Response
     */
    public function show(MovieRequest $request, MovieRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('movie::movie.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('movie::movie.show')
            ->output();
    }

    /**
     * Show the form for creating a new movie.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(MovieRequest $request, MovieRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('movie::movie.name'))
            ->view('movie::movie.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new movie.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(MovieRequest $request, MovieRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('movie::movie.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('movie/movie/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/movie/movie'))
                ->redirect();
        }

    }

    /**
     * Show movie for editing.
     *
     * @param Request $request
     * @param Model   $movie
     *
     * @return Response
     */
    public function edit(MovieRequest $request, MovieRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('movie::movie.name'))
            ->view('movie::movie.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the movie.
     *
     * @param Request $request
     * @param Model   $movie
     *
     * @return Response
     */
    public function update(MovieRequest $request, MovieRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('movie::movie.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('movie/movie/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('movie/movie/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the movie.
     *
     * @param Model   $movie
     *
     * @return Response
     */
    public function destroy(MovieRequest $request, MovieRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('movie::movie.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('movie/movie/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('movie/movie/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
